<?php

require_once 'base/GpemovimentacaoEntityBase.php';


/**
 * GpemovimentacaoEntity.
 *
 * @package Entity
 * @author  alessandrobartels
 */
class GpemovimentacaoEntity extends GpemovimentacaoEntityBase
{

    private $estrutura;
    private $vinculo;

    /**
     * Retorna a movimentacao mais recente do vinculo por estrutura.
     *
     * @param integer    $nr_org         Numero da Organizacao.
     * @param integer    $nr_estrutura   Numero da Estrutura.
     * @param integer    $nr_vinculo     Numero do Vinculo.
     * @param TkDateTime $dt_competencia Data de competencia.
     *
     *@return GpemovimentacaoEntity
     */
    public function retornaMovimentacaoRecentePorEstrutura(
        $nr_org, $nr_estrutura, $nr_vinculo, TkDateTime $dt_competencia
    ){
        $resultado = current(
            Gpemovimentacao::retornaMovimentacaoRecentePorEstrutura(
                $nr_org, $nr_estrutura, $nr_vinculo, $dt_competencia
            )
        );
        return $resultado;

    }


    /**
     * Enter description here ...
     *
     * @param integer $nrOrg                Numero da Organizacao.
     * @param integer $nrVinculo            Numero do Vinculo.
     * @param string  $ultimoDiaCompetencia Ultimo dia do mes de competencia.
     *
     * @return void
     */
    public function carregaMovimentacoes($nrOrg, $nrVinculo, $ultimoDiaCompetencia){
        $this->resetItems();
        $resultados = Gpemovimentacao::retornaMovimentacoes($nrOrg, $nrVinculo, $ultimoDiaCompetencia);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }


    /**
     * Carrega as movimentacoes do tipo informado.
     *
     * @param integer $nrOrg                Numero da Organizacao.
     * @param integer $nrVinculo            Numero do Vinculo.
     * @param integer $tipo                 Tipo da Movimentacao (Legal, Gerencial).
     * @param string  $ultimoDiaCompetencia Ultimo dia do mes de competencia.
     *
     * @return void
     */
    public function carregaMovimentacoesPorTipo($nrOrg, $nrVinculo, $tipo, $ultimoDiaCompetencia, $tipoEstrutura){
        $this->resetItems();
        $resultados = Gpemovimentacao::retornaMovimentacoesPorTipo($nrOrg, $nrVinculo, $tipo, $ultimoDiaCompetencia, $tipoEstrutura);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }

    /**
     * Carrega as movimentacoes da hierarquia.
     *
     * @param integer $nrOrg                Numero da Organizacao.
     * @param integer $nrVinculo            Numero do Vinculo.
     * @param integer $nrHierarquia                 Número da Hierarquia.
     * @param string  $ultimoDiaCompetencia Ultimo dia do mes de competencia.
     *
     * @return void
     */
    public function carregaMovimentacoesPorHierarquia($nrOrg, $nrVinculo, $nrtipoestrutura, $dtIni, $dtFim){
        $this->resetItems();
        $resultados = Gpemovimentacao::retornaMovimentacoesPorHierarquia($nrOrg, $nrVinculo, $nrtipoestrutura, $dtIni, $dtFim);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }

    public function carregaMovimentacoesPeriodo($nrOrg, $nrVinculo, $dtInicio, $dtFim){
        $this->resetItems();
        $resultados = Gpemovimentacao::retornaMovimentacoesPeriodo($nrOrg, $nrVinculo, $dtInicio, $dtFim);
        foreach ($resultados as $item) {
            $this->addItem($item, $item->getNrmovimentacao());
        }
    }


    public function verificaMovPorTipoTransferencia($nrTpMovTransf, $dtInicioMovimento, $dtRetroativa){

        foreach($this->getItems() as $item) {
            if ($item->getNrtpmovtransfm() === $nrTpMovTransf) {
                if (empty($dtRetroativa) === false) {
                    if ($item->getDtmovimentretro() === $dtRetroativa && ($item->getDtinimoviment() === $dtInicioMovimento)) {
                        return true;
                    }
                } else {
                    if ($item->getDtinimoviment() === $dtInicioMovimento) {
                        return true;
                    }
                }
            }
        }

        return false;
    }


    /**
     * Enter description here ...
     *
     * @param integer    $nrVinculo Numero do Vinculo.
     * @param TkDateTime $dtInicio  Data de inicio do periodo.
     * @param TkDateTime $dtFim     Data fim do periodo.
     *
     * @return Ambigous <multitype:, unknown, number>
     */
    public function analisaPeriodoMovimentacoes($nrVinculo, TkDateTime $dtInicio, TkDateTime $dtFim){

        $data_ini_aux = new TkDateTime();
        $data_fim_aux = new TkDateTime();

        $movimentacoes = $this->getItems();
        $diasMovimentacoes = array();
        foreach ($movimentacoes as $movimentacao) {
            if ($movimentacao->getNrvinculom() === $nrVinculo) {
                // Determina data inicio da movimentacao.
                $data_ini_aux->set($movimentacao->getDtinimoviment(), TkDateTime::FORMAT_DATE);

                // Determina data final da movimentacao.
                if ($movimentacao->getDtfimmoviment() !== null) {
                    $data_fim_aux->set($movimentacao->getDtfimmoviment(), TkDateTime::FORMAT_DATE);
                } else {
                    $data_fim_aux->set($dtFim);
                }

                if ((($data_ini_aux->isLater($dtInicio) === true)
                    ||($data_ini_aux->equals($dtInicio) === true))
                    &&(($data_ini_aux->isEarlier($dtFim) === true)
                    ||($data_ini_aux->equals($dtFim) === true))
                ) {
                    // Comecou e acabou durante.
                    $qtd_dias = $data_ini_aux->daysAfter($data_fim_aux);
                    $diasMovimentacoes[$movimentacao->getNrmovimentacao()] = $qtd_dias;
                } elseif (($data_ini_aux->isEarlier($dtInicio) === true)
                        &&(($data_fim_aux->isEarlier($dtFim) === true)
                        || ($data_fim_aux->equals($dtFim) === true))
                ) {
                    if (($data_fim_aux->isLater($dtInicio) === true)
                        ||($data_fim_aux->equals($dtInicio) === true)
                    ) {
                        // Comecou antes e acabou durante.
                        $qtd_dias = $data_ini_aux->daysAfter($data_fim_aux);
                        $diasMovimentacoes[$movimentacao->getNrmovimentacao()] = $qtd_dias;
                    }
                }
            }
        }

        return $diasMovimentacoes;

    }


    /**
     * Inclui movimentacao do vinculo.
     *
     * @param integer    $nrOrg            Numero da Organizacao.
     * @param integer    $nrTipoTransfer   Numero do Tipo de transferencia.
     * @param integer    $nrVinculom       Numero do Vinculo.
     * @param integer    $nrEstruturam     Numero da Estrutra.
     * @param string     $dtIniMoviment    Data inicial da movimentacao do vinculo.
     * @param integer    $nrTipoEstrutura  Numero do Tipo de Estrutura.
     * @param integer    $nrTpMovTransf    Numero do Tipo de Movimentacao de Transferencia.
     * @param TkDateTime $dtRetroativa     Data Retroativa.
     * @param string     $idAtualizaAcerto Identificar de atualizacao de acerto (normal = null, excluir = 1 ou incluir = 2).
     * @param string     $dtFimMovimento   Data final da movimentacao do vinculo.
     *
     * @return Instancia de GpemovimentacaoEntity.
     *
     * @author talitaorfano 13/09/2011
     *         thalesvalias 26/06/2012 - adicionado alguns parametros e funcao retorna instancia de GpemovimentacaoEntity
     */
    public function incluiMovimentacao(
        $nrOrg, $nrTipoTransfer, $nrVinculom, $nrEstruturam, $dtIniMoviment, $nrTipoEstrutura = null,
        $nrTpMovTransf = null, $dtRetroativa = null, $idAtualizaAcerto = null, $dtFimMovimento = null
    ){
        $movimentacao = new self();
        $movimentacao->setNrorg($nrOrg);
        $movimentacao->setNrtipotransfer($nrTipoTransfer);
        $movimentacao->setNrvinculom($nrVinculom);
        $movimentacao->setNrestruturam($nrEstruturam);
        $movimentacao->setDtinimoviment($dtIniMoviment);
        $movimentacao->setNrtipoestrutura($nrTipoEstrutura);
        $movimentacao->setNrtpmovtransfm($nrTpMovTransf);
        $movimentacao->setDtmovimentretro($dtRetroativa);
        $movimentacao->setIdatualizaacerto($idAtualizaAcerto);
        $movimentacao->setDtfimmoviment($dtFimMovimento);

        $movimentacao->persist();

        // Retorna movimentacao para uso no CAGED
        return $movimentacao;

    }


    /**
     * Retorna a movimentacao anterior do vinculo de acordo com a data de movimentacao informada.
     *
     * @param string  $mesCompetencia  Mes de competencia.
     * @param integer $nrOrg           Numero da organizacao.
     * @param integer $nrVinculo       Tipo do registro.
     * @param string  $dtMovimentacao  Tipo do registro.
     * @param array   $cdCagedAdmissao Tipo do registro.
     *
     * @return null || array
     *
     * @author thalesvalias 12/06/2012
     */
    public function retornaMovimentacaoAnterior($mesCompetencia, $nrOrg, $nrVinculo, $dtMovimentacao, $cdCagedAdmissao){
        $movimentacaoAnterior = Gpemovimentacao::retornaMovimentacaoAnterior(
            $mesCompetencia, $nrOrg, $nrVinculo, $dtMovimentacao, $cdCagedAdmissao
        );

        $movimentacaoAnterior = end($movimentacaoAnterior);
        return $movimentacaoAnterior;
    }

    /**
     * Retorna a ultima alteracao de ocupacao do vinculo conforme mes de competencia.
     * @static
     *
     * @param integer    $nrOrg          Numero da Organizacao.
     * @param integer    $nrVinculo      Numero do vinculo.
     *
     * @return GpemovimentacaoEntity  | null
     * @author brunoborba - 07/03/2013
     */

    public function retornaMovimentacoesVinculo($nrOrg, $nrVinculo){
        $resultado = GpeMovimentacao::retornaMovimentacoesVinculo($nrOrg, $nrVinculo);
        return $resultado;
    }

 /*   public function adicionaMovimentacao(
        $nrOrg, $nrTipoTransfer, $nrVinculom, $nrEstruturam, $dtIniMoviment, $nrTipoEstrutura = null,
        $dtRetroativa = null, $idAtualizacao = null
    ){

        $movimentacao = new self();
        $movimentacao->setNrorg($nrOrg);
        $movimentacao->setNrtipotransfer($nrTipoTransfer);
        $movimentacao->setNrvinculom($nrVinculom);
        $movimentacao->setNrestruturam($nrEstruturam);
        $movimentacao->setDtinimoviment($dtIniMoviment);
        $movimentacao->setNrtipoestrutura($nrTipoEstrutura);
        $movimentacao->setDtmovimentretro($dtRetroativa);
        $movimentacao->setIdatualizaacerto($idAtualizacao);

        return $movimentacao;

        //$this->addItem($movimentacao);
       // unset($movimentacao);
    }        */

    public function setMovimentacao($nrorg, $nrtipotransfer, $nrvinculom, $nrestruturam, $dtinimoviment,
                                       $nrtipoestrutura, $nrtpmovtransfm, $dtretroativa, $idatualizaacerto,
                                       $dtfimmovimento)
    {
        $this->setNrorg($nrorg);
        $this->setNrtipotransfer($nrtipotransfer);
        $this->setNrvinculom($nrvinculom);
        $this->setNrestruturam($nrestruturam);
        $this->setDtinimoviment($dtinimoviment);
        $this->setNrtipoestrutura($nrtipoestrutura);
        $this->setNrtpmovtransfm($nrtpmovtransfm);
        $this->setDtmovimentretro($dtretroativa);
        $this->setIdatualizaacerto($idatualizaacerto);
        $this->setDtfimmoviment($dtfimmovimento);
    }

    /**
     * @return \GpevinculomEntity
     */
    public function getVinculo() {
        if (!$this->vinculo) {
            $this->vinculo = Gpevinculom::getVinculo($this->getNrorg(), $this->getNrvinculom());
        }
        return $this->vinculo;
    }

    /**
     * @return EstruturamEntity
     */
    public function getEstrutura() {
        if (!$this->estrutura) {
            $this->estrutura = EstruturamEntity::getEntityByWhere(
                'NRORG = ? AND NRESTRUTURAM = ?', array($this->getNrorg(), $this->getNrestruturam())
            );
        }
        return $this->estrutura;
    }

}